@@ -454,13 +454,10 @@ class FindReplaceWidget(QtGui.QFrame):
         flags = QtGui.QTextDocument.FindFlags()
         if self._caseCheck.isChecked():
             flags |= QtGui.QTextDocument.FindCaseSensitively
-
-        if self._wholeWord.isChecked():
-            flags |= QtGui.QTextDocument.FindWholeWords
-        
         if not forward:
             flags |= QtGui.QTextDocument.FindBackward
-
+        #if self._wholeWord.isChecked():
+        #    flags |= QtGui.QTextDocument.FindWholeWords
         
         # focus
         self.selectFindText()
@@ -473,6 +470,13 @@ class FindReplaceWidget(QtGui.QFrame):
             needle = QtCore.QRegExp(needle,
                 QtCore.Qt.CaseSensitive if self._caseCheck.isChecked() else
                 QtCore.Qt.CaseInsensitive)
+        elif self._wholeWord.isChecked():
+            # Use regexp, because the default begaviour does not find
+            # whole words correctly, see issue #276
+            # it should *not* find this in this_word
+            needle = QtCore.QRegExp(r'\b' + needle + r'\b',
+                QtCore.Qt.CaseSensitive if self._caseCheck.isChecked() else
+                QtCore.Qt.CaseInsensitive)
         
         # estblish start position
         cursor = editor.textCursor()
@@ -530,7 +534,7 @@ class FindReplaceWidget(QtGui.QFrame):
             original = original.lower()
         
         # replace
-        #TODO: this line does not work for regexp-search!
+        #TODO: < line does not work for regexp-search!
         if original and original == needle:
             cursor.insertText( replacement )
         
@@ -995,22 +999,31 @@ class EditorTabs(QtGui.QWidget):
         self.breakPointsChanged.emit(self._breakPoints)
     
     
-    def setDebugLineIndicator(self, filename, linenr):
+    def setDebugLineIndicators(self, *filename_linenr):
         """ Set the debug line indicator. There is one indicator
         global to IEP, corresponding to the last shell for which we
         received the indicator.
         """
-        # Use normcase to compare file names. OOn Windows we can have
-        # "c:\.." and "C:\..." for instance
-        filename = os.path.normcase(filename)
-        for editor in self:
+        if len(filename_linenr) and filename_linenr[0] is None:
+            filename_linenr = []
+        
+        # Normalize case
+        filename_linenr = [(os.path.normcase(i[0]), int(i[1])) for i in filename_linenr]
+        
+        for item in self._tabs.items():
+            # Prepare
+            editor = item._editor
             fname = editor._filename or editor._name
-            if os.path.normcase(fname) == filename:
-                editor.setDebugLineIndicator(linenr)
-            else:
-                editor.setDebugLineIndicator(None)
-    
-    
+            fname = os.path.normcase(fname)
+            # Reset
+            editor.setDebugLineIndicator(None)
+            # Set
+            for filename, linenr in filename_linenr:
+                if fname == filename:
+                    active = (filename, linenr) == filename_linenr[-1]
+                    editor.setDebugLineIndicator(linenr, active)
+            
+
     ## Loading ad saving files
     
     def dragEnterEvent(self, event):
