@@ -478,6 +478,8 @@ class FileMenu(Menu):
                 icons.page_delete, iep.editors.closeFile),
             self.addItem(translate("menu", "Close all ::: Close all files."),
                 icons.page_delete_all, iep.editors.closeAllFiles),  
+            self.addItem(translate("menu", "Export to PDF ::: Export current file to PDF (e.g. for printing)."),
+                None, self._print),  
             ]
         
         # Build file properties stuff
@@ -517,7 +519,7 @@ class FileMenu(Menu):
             # Update parser
             parserName = 'None'
             if editor.parser():
-                parserName = editor.parser().name()
+                parserName = editor.parser().name() or 'None'
             self._parserMenu.setCheckedOption(None, parserName )
             # Update line ending
             self._lineEndingMenu.setCheckedOption(None, editor.lineEndingsHumanReadable)
@@ -575,6 +577,29 @@ class FileMenu(Menu):
         editor = iep.editors.getCurrentEditor()
         if editor is not None:
             editor.encoding = value
+    
+    def _print(self):
+        editor = iep.editors.getCurrentEditor()
+        if editor is not None:
+            printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
+            if True:
+                filename = QtGui.QFileDialog.getSaveFileName(None, 
+                        'Export PDF', os.path.expanduser("~"), "*.pdf *.ps")
+                if isinstance(filename, tuple): # PySide
+                    filename = filename[0]
+                if not filename:
+                    return
+                printer.setOutputFileName(filename)
+            else:
+                d = QtGui.QPrintDialog(printer)
+                d.setWindowTitle('Print code')
+                d.setOption(d.PrintSelection, editor.textCursor().hasSelection())
+                d.setOption(d.PrintToFile, True)
+                ok = d.exec_()
+                if ok != d.Accepted:
+                    return
+            # Print
+            editor.print_(printer)
 
 
 # todo: move to matching brace
@@ -819,7 +844,7 @@ class ShellMenu(Menu):
         return [
             self.addItem(translate("menu", 'Debug next: proceed until next line'), 
                 icons.debug_next, self._debugAction, "NEXT"),
-            self.addItem(translate("menu", 'Debug step: proceed one step'), 
+            self.addItem(translate("menu", 'Debug step into: proceed one step'), 
                 icons.debug_step, self._debugAction, "STEP"),
             self.addItem(translate("menu", 'Debug return: proceed until returns'), 
                 icons.debug_return, self._debugAction, "RETURN"),
@@ -1122,26 +1147,30 @@ class RunMenu(Menu):
     def build(self):
         icons = iep.icons
         
-        self.addItem(translate("menu", 'Run selection ::: Run the current editor\'s selected lines, selected words on the current line, or current line if there is no selection.'), 
+        self.addItem(translate("menu", 'Run file as script ::: Restart and run the current file as a script.'), 
+            icons.run_file_script, self._runFile, (True, False))
+        self.addItem(translate("menu", 'Run main file as script ::: Restart and run the main file as a script.'), 
+            icons.run_mainfile_script, self._runFile, (True, True))
+        
+        self.addSeparator()
+        
+        self.addItem(translate("menu", 'Execute selection ::: Execute the current editor\'s selected lines, selected words on the current line, or current line if there is no selection.'), 
             icons.run_lines, self._runSelected)
-        self.addItem(translate("menu", 'Run cell ::: Run the current editors\'s cell in the current shell.'), 
+        self.addItem(translate("menu", 'Execute cell ::: Execute the current editors\'s cell in the current shell.'), 
             icons.run_cell, self._runCell)
-        self.addItem(translate("menu", 'Run cell and advance ::: Run the current editors\'s cell and advance to the next cell.'), 
+        self.addItem(translate("menu", 'Execute cell and advance ::: Execute the current editors\'s cell and advance to the next cell.'), 
             icons.run_cell, self._runCellAdvance)
         #In the _runFile calls, the parameter specifies (asScript, mainFile)
-        self.addItem(translate("menu", 'Run file ::: Run the current file in the current shell.'), 
+        self.addItem(translate("menu", 'Execute file ::: Execute the current file in the current shell.'), 
             icons.run_file, self._runFile,(False, False))
-        self.addItem(translate("menu", 'Run main file ::: Run the main file in the current shell.'), 
+        self.addItem(translate("menu", 'Execute main file ::: Execute the main file in the current shell.'), 
             icons.run_mainfile, self._runFile,(False, True))
+        
         self.addSeparator()
-        self.addItem(translate("menu", 'Run file as script ::: Run the current file as a script.'), 
-            icons.run_file_script, self._runFile, (True, False))
-        self.addItem(translate("menu", 'Run main file as script ::: Run the main file as a script.'), 
-            icons.run_mainfile_script, self._runFile, (True, True))
-        self.addSeparator()
+        
         self.addItem(translate("menu", 'Help on running code ::: Open the IEP wizard at the page about running code.'), 
             icons.information, self._showHelp)
-
+    
     
     def _showHelp(self):
         """ Show more information about ways to run code. """
@@ -1385,6 +1414,10 @@ class HelpMenu(Menu):
     
     def build(self):
         icons = iep.icons
+        issues_url = "https://bitbucket.org/iep-project/iep/issues/"
+        if iep.pyzo_mode:
+            issues_url = "http://pyzo.org/issues.html"
+        
         if iep.pyzo_mode:
             self.addUrlItem(translate("menu", "Pyzo Website ::: Open the Pyzo website in your browser."), 
                 icons.help, "http://www.pyzo.org")
@@ -1393,7 +1426,7 @@ class HelpMenu(Menu):
         self.addUrlItem(translate("menu", "Ask a question ::: Need help?"), 
             icons.comments, "http://pyzo.org/community.html#discussion-fora-and-email-lists")
         self.addUrlItem(translate("menu", "Report an issue ::: Did you found a bug in IEP, or do you have a feature request?"), 
-            icons.error_add, "http://pyzo.org/issues.html")
+            icons.error_add, issues_url)
         self.addSeparator()
         self.addItem(translate("menu", "IEP wizard ::: Get started quickly."), 
             icons.wand, self._showIepWizard)
