@@ -20,6 +20,7 @@ from pyzolib.qt import QtCore, QtGui
 Qt = QtCore.Qt
 
 import os, sys, time, subprocess
+import re
 import yoton
 import iep
 from pyzolib import ssdf
@@ -159,28 +160,36 @@ class ShellHighlighter(Highlighter):
             else:
                 pos1, pos2 = 0, commandCursor.positionInBlock()
             
+            # Check if we should *not* format this line.
+            # This is the case for special "executing" text
+            # A bit of a hack though ... is there a better way to signal this?
+            specialinput = (not atCurrentPrompt) and line[pos2:].startswith('(executing ')
+            
             self.setCurrentBlockState(0)
-            for token in parser.parseLine(line, previousState):
-                # Handle block state
-                if isinstance(token, parsers.BlockState):
-                    self.setCurrentBlockState(token.state)
-                else:
-                    # Get format
-                    try:
-                        format = nameToFormat(token.name).textCharFormat
-                    except KeyError:
-                        #print(repr(nameToFormat(token.name)))
-                        continue
-                    # Set format                    
-                    #format.setFontWeight(99)
-                    if token.start >= pos2:
-                        self.setFormat(token.start,token.end-token.start,format)
+            if specialinput:
+                pass # Let the kernel decide formatting
+            else:
+                for token in parser.parseLine(line, previousState):
+                    # Handle block state
+                    if isinstance(token, parsers.BlockState):
+                        self.setCurrentBlockState(token.state)
+                    else:
+                        # Get format
+                        try:
+                            format = nameToFormat(token.name).textCharFormat
+                        except KeyError:
+                            #print(repr(nameToFormat(token.name)))
+                            continue
+                        # Set format                    
+                        #format.setFontWeight(75)
+                        if token.start >= pos2:
+                            self.setFormat(token.start,token.end-token.start,format)
                 
             # Set prompt to bold
             if atCurrentPrompt:
                 format = QtGui.QTextCharFormat()
-                format.setFontWeight(99)
-                self.setFormat(pos1, pos2-pos1 ,format)
+                format.setFontWeight(75)
+                self.setFormat(pos1, pos2-pos1, format)
         
         #Get the indentation setting of the editors
         indentUsingSpaces = self._codeEditor.indentUsingSpaces()
@@ -272,6 +281,11 @@ class BaseShell(BaseTextCtrl):
         # is before the prompt
         self.cursorPositionChanged.connect(self.onCursorPositionChanged)
     
+        self.setFocusPolicy(Qt.TabFocus) # See remark at mousePressEvent
+    
+    ## Cursor stuff
+    
+    
     
     def onCursorPositionChanged(self):
         #If the end of the selection (or just the cursor if there is no selection)
@@ -298,9 +312,24 @@ class BaseShell(BaseTextCtrl):
     
     
     def mousePressEvent(self, event):
-        """ Disable right MB and middle MB (which pastes by default). """
+        """ 
+        - Disable right MB and middle MB (which pastes by default). 
+        - Focus policy
+            If a user clicks this shell, while it has no focus, we do
+            not want the cursor position to change (since generally the
+            user clicks the shell to give it the focus). We do this by
+            setting the focus-policy to Qt::TabFocus, and we give the
+            widget its focus manually from the mousePressedEvent event
+            handler
+            
+        """
+        
+        if not self.hasFocus():
+            self.setFocus()  
+            return
+        
         if event.button() != QtCore.Qt.MidButton:
-            BaseTextCtrl.mousePressEvent(self, event)
+            super().mousePressEvent(event)
     
     
     def contextMenuEvent(self, event):
@@ -308,6 +337,107 @@ class BaseShell(BaseTextCtrl):
         pass
     
     
+    def mouseDoubleClickEvent(self, event):
+        BaseTextCtrl.mouseDoubleClickEvent(self, event)
+        self._handleClickOnFilename(event.pos())
+    
+    
+    def _handleClickOnFilename(self, mousepos):
+        """ Check whether the text that is clicked is a filename
+        and open the file in the editor. If a line number can also be 
+        detected, open the file at that line number.
+        """
+        
+        # Get cursor and its current pos
+        cursor = self.cursorForPosition(mousepos)
+        ocursor = QtGui.QTextCursor(cursor)  # Make a copy to use below
+        pos = cursor.positionInBlock()
+        
+        # Get line of text for the cursor
+        cursor.movePosition(cursor.EndOfBlock, cursor.MoveAnchor)
+        cursor.movePosition(cursor.StartOfBlock, cursor.KeepAnchor)
+        line = cursor.selectedText()
+        if len(line) > 1024:
+            return  # safety
+        
+        # Get the word that is clicked. Qt's cursor.StartOfWord does not
+        # work for filenames.
+        line = line.replace('"', ' ')  # IEP uses " around filenames
+        before = line[:pos].split(' ')[-1]
+        after = line[pos:].split(' ')[0]
+        word = before + after
+        
+        # Check if it looks like a filename, quit if it does not
+        if len(word) < 4:
+            return
+        elif not ('/' in word or '\\' in word):
+            return
+        #
+        if sys.platform.startswith('win'):
+            if word[1] != ':':
+                return
+        else:
+           if not word.startswith('/'):
+               return
+        #
+        filename = word
+        
+        # Split in parts for getting line number
+        line = line[pos+len(after):]
+        line = line.replace(',', ' ')
+        parts = [p for p in line.split(' ') if p]
+        # Iterate over parts
+        linenr = None
+        for i, part in enumerate(parts):
+            if part in ('line', 'linenr', 'lineno'):
+                try:
+                    linenr = int(parts[i+1])
+                except IndexError:
+                    pass  # no more parts
+                except ValueError:
+                    pass  # not an integer
+                else:
+                    break
+        
+        # Try again IPython style
+        # IPython shows a few lines with the active line indicated by an arrow
+        if linenr is None:
+            for i in range(4):
+                cursor.movePosition(cursor.NextBlock, cursor.MoveAnchor)
+                cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
+                line = cursor.selectedText()
+                if len(line) > 1024:
+                    continue  # safety
+                if not line.startswith('-'):
+                    continue
+                parts = line.split(' ', 2)
+                if parts[0] in ('->', '-->', '--->', '---->', '----->'):
+                    try:
+                        linenr = int(parts[1].strip())
+                    except IndexError:
+                        pass  # too few parts
+                    except ValueError:
+                        pass  # not an integer
+                    else:
+                        break
+        
+        # Select word here (in shell)
+        cursor = ocursor
+        cursor.movePosition(cursor.Left, cursor.MoveAnchor, len(before))
+        cursor.movePosition(cursor.Right, cursor.KeepAnchor, len(word))
+        self.setTextCursor(cursor)
+        
+        # Try opening the file (at the line number if we have one)
+        result = iep.editors.loadFile(filename)
+        if result and linenr is not None:
+                editor = result._editor
+                editor.gotoLine(linenr)
+                cursor = editor.textCursor()
+                cursor.movePosition(cursor.StartOfBlock)
+                cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
+                editor.setTextCursor(cursor)
+    
+    
     ##Indentation: override code editor behaviour
     def indentSelection(self):
         pass
@@ -612,19 +742,13 @@ class BaseShell(BaseTextCtrl):
             self._cursor1.setKeepPositionOnInsert(False)
             self._cursor2.setKeepPositionOnInsert(False)
             text = self._handleBackspaces(text)
-            if len(text) < 1024:
-                # Insert text 
-                self._cursor1.insertText(text, format)
-            else:
-                # Insert per line (very long lines are split in smaller ones)
-                for line in self._splitLinesForPrinting(text):
-                    self._cursor1.insertText(line, format)
+            self._insertText(self._cursor1, text, format)
         elif prompt == 1:
             # Insert command text after prompt, prompt becomes null (input)
             self._lastCommandCursor.setPosition(self._cursor2.position())
             self._cursor1.setKeepPositionOnInsert(False)
             self._cursor2.setKeepPositionOnInsert(False)
-            self._cursor2.insertText(text, format)
+            self._insertText(self._cursor2, text, format)
             self._cursor1.setPosition(self._cursor2.position(), A_MOVE)
         elif prompt == 2 and text == '\b':
             # Remove prompt (used when closing the kernel)
@@ -636,7 +760,7 @@ class BaseShell(BaseTextCtrl):
             self._cursor1.setPosition(self._cursor2.position(), A_MOVE)
             self._cursor1.setKeepPositionOnInsert(True)
             self._cursor2.setKeepPositionOnInsert(False)
-            self._cursor1.insertText(text, format)
+            self._insertText(self._cursor1, text, format)
         
         # Reset cursor states for the user to type his/her commands
         self._cursor1.setKeepPositionOnInsert(True)
@@ -653,6 +777,89 @@ class BaseShell(BaseTextCtrl):
             sb.setValue(sb.value()-n) 
     
     
+    def _insertText(self, cursor, text, format):
+        """ Insert text at the given cursor, and with the given format.
+        This function processes ANSI escape code for formatting and
+        colorization: http://en.wikipedia.org/wiki/ANSI_escape_code
+        """
+        
+        # If necessary, make a new cursor that moves along. We insert
+        # the text in pieces, so we need to move along with the text!
+        if cursor.keepPositionOnInsert():
+            cursor = QtGui.QTextCursor(cursor)
+            cursor.setKeepPositionOnInsert(False)
+        
+        # Init. We use the solarised color theme
+        pattern = r'\x1b\[(.*?)m'
+        #CLRS = ['#000', '#F00', '#0F0', '#FF0', '#00F', '#F0F', '#0FF', '#FFF']
+        CLRS = ['#657b83', '#dc322f', '#859900', '#b58900', '#268bd2', 
+                '#d33682', '#2aa198', '#eee8d5']
+        pendingtext = ''
+        i0 = 0
+        
+        
+        for match in re.finditer(pattern, text):
+            
+            # Insert pending text with the current format
+            # Also update indices
+            i1, i2 = match.span()
+            cursor.insertText(text[i0:i1], format)
+            i0 = i2
+            
+            # The formay that we are now going to parse should apply to
+            # the text that follow it ...
+            
+            # Get parameters
+            try:
+                params = [int(i) for i in match.group(1).split(';')]
+            except ValueError:
+                params = []
+            if not params:
+                params = [0]
+            
+            # Process
+            for param in params:
+                if param == 0:
+                    format = QtGui.QTextCharFormat()
+                elif param == 1:
+                    format.setFontWeight(75)  # Bold
+                elif param == 2:
+                    format.setFontWeight(25)  # Faint
+                elif param == 3:
+                    format.setFontItalic(True)  # Italic
+                elif param == 4:
+                    format.setFontUnderline(True)  # Underline
+                #
+                elif param == 22:
+                    format.setFontWeight(50)  # Not bold or faint
+                elif param == 23:
+                    format.setFontItalic(False)  # Not italic
+                elif param == 24:
+                    format.setFontUnderline(False)  # Not underline
+                #
+                elif 30 <= param <= 37:  # Set foreground color
+                    clr = CLRS[param-30]
+                    format.setForeground(QtGui.QColor(clr))
+                elif 40 <= param <= 47:
+                    pass # Cannot set background text in QPlainTextEdit
+                #
+                else:
+                    pass  # Not supported
+            
+        else:
+            # At the end, process the remaining text
+            text = text[i0:]
+            
+            # Process very long text more efficiently.
+            # Insert per line (very long lines are split in smaller ones)
+            if len(text) > 1024:
+                for line in self._splitLinesForPrinting(text):
+                    cursor.insertText(line, format)
+            else:
+                cursor.insertText(text, format)
+
+
+    
     ## Executing stuff
     
     def processLine(self, line=None, execute=True):
@@ -830,6 +1037,9 @@ class PythonShell(BaseShell):
         self._brokerConnection = ct.connect('localhost:%i'%slot)
         self._brokerConnection.closed.bind(self._onConnectionClose)
         
+        # Force updating of breakpoints
+        iep.editors.updateBreakPoints()
+        
         # todo: see polling vs events
 #         # Detect incoming messages 
 #         for c in [self._strm_out, self._strm_err, self._strm_raw, 
@@ -1017,6 +1227,10 @@ class PythonShell(BaseShell):
         """ executeCommand(text)
         Execute one-line command in the remote Python session. 
         """
+        
+        # Ensure edit line is selected (to reset scrolling to end)
+        self.ensureCursorAtEditLine()
+        
         self._ctrl_command.send(text)
     
     
@@ -1078,6 +1292,10 @@ class PythonShell(BaseShell):
             lines2.append( line )
         
         
+        # Ensure edit line is selected (to reset scrolling to end)
+        self.ensureCursorAtEditLine()
+
+        
         # Send message
         text = "\n".join(lines2)
         msg = {'source':text, 'fname':fname, 'lineno':lineno, 'cellName': cellName}
@@ -1150,11 +1368,27 @@ class PythonShell(BaseShell):
         action = self._strm_action.recv(False)
         if action:
             if action.startswith('open '):
-                fname = action.split(' ',1)[1]
-                iep.editors.loadFile(fname)
+                parts = action.split(' ')
+                parts.pop(0)
+                try:
+                    linenr = int(parts[0])
+                    parts.pop(0)
+                except ValueError:
+                    linenr = None
+                fname = ' '.join(parts)
+                editor = iep.editors.loadFile(fname)
+                if editor and linenr:
+                    editor._editor.gotoLine(linenr)
             else:
                 print('Unkown action: %s' % action)
         
+        # ----- status
+        
+        # Do not update status when the kernel is not really up and running
+        # self._version is set when the startup info is received
+        if not self._version:
+            return
+        
         # Update status
         state = self._stat_interpreter.recv()
         if state != self._state:
@@ -1173,6 +1407,9 @@ class PythonShell(BaseShell):
         Send a Keyboard interrupt signal to the main thread of the 
         remote process. 
         """
+        # Ensure edit line is selected (to reset scrolling to end)
+        self.ensureCursorAtEditLine()
+        
         self._ctrl_broker.send('INT')
     
     
@@ -1182,7 +1419,10 @@ class PythonShell(BaseShell):
         Args can be a filename, to execute as a script as soon as the
         shell is back up.
         """
-        
+
+        # Ensure edit line is selected (to reset scrolling to end)
+        self.ensureCursorAtEditLine()
+                
         # Get info
         info = finishKernelInfo(self._info, scriptFile)
         
@@ -1201,6 +1441,10 @@ class PythonShell(BaseShell):
         To be notified of the termination, connect to the "terminated"
         signal of the shell.
         """
+        # Ensure edit line is selected (to reset scrolling to end)
+        self.ensureCursorAtEditLine()
+        
+
         self._ctrl_broker.send('TERM')
     
     
