@@ -39,6 +39,8 @@ class ShellInfo_name(ShellInfoLineEdit):
     def __init__(self, *args, **kwargs):
         ShellInfoLineEdit.__init__(self, *args, **kwargs)
         self.editingFinished.connect(self.onValueChanged)
+        t = translate('shell', 'name ::: The name of this configuration.')
+        self.setPlaceholderText(t.tt)
     
     
     def setTheText(self, value):
@@ -47,7 +49,7 @@ class ShellInfo_name(ShellInfoLineEdit):
     
     
     def onValueChanged(self): 
-        self.parent().setTabTitle(self.getTheText())
+        self.parent().parent().parent().setTabTitle(self.getTheText())
 
 
 
@@ -121,6 +123,28 @@ class ShellInfo_exe(QtGui.QComboBox):
             guicombobox.setTheText(defaultGui)
 
 
+class ShellInfo_ipython(QtGui.QCheckBox):
+    
+    def __init__(self, parent):
+        QtGui.QCheckBox.__init__(self, parent)
+        t = translate('shell', 'ipython ::: Use IPython shell if available.')
+        self.setText(t.tt)
+        # Default is True
+        self.setChecked(True)
+    
+    def setTheText(self, value):
+        if value.lower() in ['no', 'false']:
+            self.setChecked(False)
+        else: 
+            self.setChecked(True)  # Also for empty string; default is True
+    
+    def getTheText(self):
+        if self.isChecked():
+            return 'yes'
+        else:
+            return 'no'
+
+
 class ShellInfo_gui(QtGui.QComboBox):
     
     # For (backward) compatibility
@@ -166,8 +190,9 @@ class ShellInfo_gui(QtGui.QComboBox):
 class ShellinfoWithSystemDefault(QtGui.QVBoxLayout):
     
     DISABLE_SYSTEM_DEFAULT = sys.platform == 'darwin' 
+    SYSTEM_VALUE = ''
     
-    def __init__(self, parent, widget, systemValue):
+    def __init__(self, parent, widget):
         # Do not pass parent, because is a sublayout
         QtGui.QVBoxLayout.__init__(self) 
         
@@ -185,9 +210,6 @@ class ShellinfoWithSystemDefault(QtGui.QVBoxLayout):
         # The actual value of this shell config attribute
         self._value = ''
         
-        # The value of self._value if the system default is selected
-        self._systemValue = systemValue
-        
         # A buffered version, so that clicking the text box does not
         # remove the value at once
         self._bufferedValue = ''
@@ -201,7 +223,7 @@ class ShellinfoWithSystemDefault(QtGui.QVBoxLayout):
     def onCheckChanged(self, state):
         if state:
             self._bufferedValue = self._value
-            self.setTheText(self._systemValue)
+            self.setTheText(self.SYSTEM_VALUE)
         else:
             self.setTheText(self._bufferedValue)
     
@@ -213,7 +235,7 @@ class ShellinfoWithSystemDefault(QtGui.QVBoxLayout):
             self._edit.setReadOnly(False)
             self.setWidgetText(value)
         
-        elif value != self._systemValue:
+        elif value != self.SYSTEM_VALUE:
             # Value given, enable edit
             self._check.setChecked(False)
             self._edit.setReadOnly(False)
@@ -238,16 +260,18 @@ class ShellinfoWithSystemDefault(QtGui.QVBoxLayout):
 
 class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
     
+    SYSTEM_VALUE = '$PYTHONPATH'
+    
     def __init__(self, parent):
         
         # Create sub-widget
         self._edit = QtGui.QTextEdit(parent)
         self._edit.setMaximumHeight(80)
-        self._edit.setMinimumWidth(400)
+        self._edit.setMinimumWidth(200)
         self._edit.textChanged.connect(self.onEditChanged)
         
         # Instantiate
-        ShellinfoWithSystemDefault.__init__(self, parent, self._edit, '$PYTHONPATH') 
+        ShellinfoWithSystemDefault.__init__(self, parent, self._edit) 
     
     
     def getWidgetText(self):
@@ -257,63 +281,233 @@ class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
     def setWidgetText(self, value=None):
         if value is None:
             pp = os.environ.get('PYTHONPATH','')
-            pp = pp.replace(os.pathsep, '\n  ').strip()
-            value = '$PYTHONPATH (\n  %s\n)' % pp
-        
+            pp = pp.replace(os.pathsep, '\n').strip()
+            value = '$PYTHONPATH:\n%s\n' % pp
         self._edit.setText(value)
 
 
 
-class ShellInfo_startupScript(ShellinfoWithSystemDefault):
+# class ShellInfo_startupScript(ShellinfoWithSystemDefault):
+#     
+#     SYSTEM_VALUE = '$PYTHONSTARTUP'
+#     
+#     def __init__(self, parent):
+#         
+#         # Create sub-widget
+#         self._edit = QtGui.QLineEdit(parent)
+#         self._edit.textEdited.connect(self.onEditChanged)
+#         
+#         # Instantiate
+#         ShellinfoWithSystemDefault.__init__(self, parent, self._edit) 
+#     
+#     
+#     def getWidgetText(self):
+#         return self._edit.text()
+#     
+#     
+#     def setWidgetText(self, value=None):
+#         if value is None:
+#             pp = os.environ.get('PYTHONSTARTUP','').strip()
+#             if pp:          
+#                 value = '$PYTHONSTARTUP: "%s"' % pp
+#             else:
+#                 value = '$PYTHONSTARTUP: None'
+#         
+#         self._edit.setText(value)
+
+
+
+class ShellInfo_startupScript(QtGui.QVBoxLayout):
+    
+    DISABLE_SYSTEM_DEFAULT = sys.platform == 'darwin' 
+    SYSTEM_VALUE = '$PYTHONSTARTUP'
     
     def __init__(self, parent):
+        # Do not pass parent, because is a sublayout
+        QtGui.QVBoxLayout.__init__(self) 
         
         # Create sub-widget
-        self._edit = QtGui.QLineEdit(parent)
-        self._edit.textEdited.connect(self.onEditChanged)
+        self._edit1 = QtGui.QLineEdit(parent)
+        self._edit1.textEdited.connect(self.onEditChanged)
+        if sys.platform.startswith('win'):
+            self._edit1.setPlaceholderText('C:\\path\\to\\script.py')
+        else:
+            self._edit1.setPlaceholderText('/path/to/script.py')
+        #
+        self._edit2 = QtGui.QTextEdit(parent)
+        self._edit2.setMaximumHeight(80)
+        self._edit2.setMinimumWidth(200)
+        self._edit2.textChanged.connect(self.onEditChanged)
         
-        # Instantiate
-        ShellinfoWithSystemDefault.__init__(self, parent, self._edit, '$PYTHONSTARTUP') 
+        # Layout
+        self.setSpacing(1)
+        self.addWidget(self._edit1)
+        self.addWidget(self._edit2)
+        
+        # Create radio widget for system default
+        t = translate('shell', 'Use system default')
+        self._radio_system = QtGui.QRadioButton(t, parent)
+        self._radio_system.toggled.connect(self.onCheckChanged)
+        self.addWidget(self._radio_system)
+        if self.DISABLE_SYSTEM_DEFAULT:
+            self._radio_system.hide()
+        
+        # Create radio widget for file
+        t = translate('shell', 'File to run at startup')
+        self._radio_file = QtGui.QRadioButton(t, parent)
+        self._radio_file.toggled.connect(self.onCheckChanged)
+        self.addWidget(self._radio_file)
+        
+        # Create radio widget for code
+        t = translate('shell', 'Code to run at startup')
+        self._radio_code = QtGui.QRadioButton(t, parent)
+        self._radio_code.toggled.connect(self.onCheckChanged)
+        self.addWidget(self._radio_code)
+        
+        # The actual value of this shell config attribute
+        self._value = ''
+        
+        # A buffered version, so that clicking the text box does not
+        # remove the value at once
+        self._valueFile = ''
+        self._valueCode = '\n'
     
     
-    def getWidgetText(self):
-        return self._edit.text()
+    def onEditChanged(self):
+        if self._radio_file.isChecked():
+            self._value = self._valueFile = self._edit1.text().strip()
+        elif self._radio_code.isChecked():
+            # ensure newline!
+            self._value = self._valueCode = self._edit2.toPlainText().strip() + '\n'
     
     
-    def setWidgetText(self, value=None):
-        if value is None:
+    def onCheckChanged(self, state):
+        if self._radio_system.isChecked():
+            self.setTheText(self.SYSTEM_VALUE)
+        elif self._radio_file.isChecked():
+            self.setTheText(self._valueFile)
+        elif self._radio_code.isChecked():
+            self.setTheText(self._valueCode)
+    
+    
+    def setTheText(self, value):
+        self.setWidgetText(value, True)
+        
+    
+    def setWidgetText(self, value, init=False):
+        self._value = value
+        
+        if value == self.SYSTEM_VALUE and not self.DISABLE_SYSTEM_DEFAULT:
+            # System default
+            if init:
+                self._radio_system.setChecked(True)
             pp = os.environ.get('PYTHONSTARTUP','').strip()
             if pp:          
-                value = '$PYTHONSTARTUP ("%s")' % pp
+                value = '$PYTHONSTARTUP: "%s"' % pp
             else:
-                value = '$PYTHONSTARTUP (is empty)'
+                value = '$PYTHONSTARTUP: None'
+            #
+            self._edit1.setReadOnly(True)
+            self._edit1.show()
+            self._edit2.hide()
+            self._edit1.setText(value)
         
-        self._edit.setText(value)
+        elif not '\n' in value:
+            # File
+            if init:
+                self._radio_file.setChecked(True)
+            self._edit1.setReadOnly(False)
+            self._edit1.show()
+            self._edit2.hide()
+            self._edit1.setText(value)
+            
+        
+        else:
+            # Code
+            if init:
+                self._radio_code.setChecked(True)
+            self._edit1.hide()
+            self._edit2.show()
+            self._edit2.setText(value)
+    
+    
+    def getTheText(self):
+        return self._value
 
 
 
 class ShellInfo_startDir(ShellInfoLineEdit):
-    pass
+    def __init__(self, parent):
+        ShellInfoLineEdit.__init__(self, parent)
+        if sys.platform.startswith('win'):
+            self.setPlaceholderText('C:\\path\\to\\your\\python\\modules')
+        else:
+            self.setPlaceholderText('/path/to/your/python/modules')
+
+
+
+class ShellInfo_argv(ShellInfoLineEdit):
+    def __init__(self, parent):
+        ShellInfoLineEdit.__init__(self, parent)
+        self.setPlaceholderText('arg1 arg2 "arg with spaces"')
+
+
+
+class ShellInfo_environ(QtGui.QTextEdit):
+    EXAMPLE = 'EXAMPLE_VAR1=value1\nEXAMPLE_VAR2=value2'
+    
+    def __init__(self, parent):
+        QtGui.QTextEdit.__init__(self, parent)
+        self.setText(self.EXAMPLE)
+    
+    def _cleanText(self, txt):
+        return '\n'.join([line.strip() for line in txt.splitlines()])
+    
+    def setTheText(self, value):
+        value = self._cleanText(value)
+        if value:
+            self.setText(value)
+        else:
+            self.setText(self.EXAMPLE)
+    
+    def getTheText(self):
+        value = self.toPlainText()
+        value = self._cleanText(value)
+        if value == self.EXAMPLE:
+            return ''
+        else:
+            return value
+
 
 
 ## The dialog class and container with tabs
 
 
-class ShellInfoTab(QtGui.QWidget):
+class ShellInfoTab(QtGui.QScrollArea):
     
     INFO_KEYS = [   translate('shell', 'name ::: The name of this configuration.'), 
                     translate('shell', 'exe ::: The Python executable.'), 
+                    translate('shell', 'ipython ::: Use IPython shell if available.'), 
                     translate('shell', 'gui ::: The GUI toolkit to integrate (for interactive plotting, etc.).'), 
                     translate('shell', 'pythonPath ::: A list of directories to search for modules and packages. Write each path on a new line, or separate with the default seperator for this OS.'), 
                     translate('shell', 'startupScript ::: The script to run at startup (not in script mode).'), 
-                    translate('shell', 'startDir ::: The start directory (not in script mode).')
+                    translate('shell', 'startDir ::: The start directory (not in script mode).'),
+                    translate('shell', 'argv ::: The command line arguments (sys.argv).'),
+                    translate('shell', 'environ ::: Extra environment variables (os.environ).'),
                 ]
     
     def __init__(self, parent):
-        QtGui.QWidget.__init__(self, parent)
+        QtGui.QScrollArea.__init__(self, parent)
         
-        # Create layout
-        self._formLayout = QtGui.QFormLayout(self)
+        # Init the scroll area
+        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
+        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
+        self.setWidgetResizable(True)
+        self.setFrameShape(QtGui.QFrame.NoFrame);
+        
+        # Create widget and a layout
+        self._content = QtGui.QWidget(parent)
+        self._formLayout = QtGui.QFormLayout(self._content)
         
         # Collect classes of widgets to instantiate
         classes = []
@@ -326,10 +520,10 @@ class ShellInfoTab(QtGui.QWidget):
         self._shellInfoWidgets = {}
         for t, cls in classes:
             # Instantiate and store
-            instance = cls(self)
+            instance = cls(self._content)
             self._shellInfoWidgets[t.key] = instance
             # Create label 
-            label = QtGui.QLabel(t, self)
+            label = QtGui.QLabel(t, self._content)
             label.setToolTip(t.tt)
             # Add to layout
             self._formLayout.addRow(label, instance)
@@ -337,8 +531,8 @@ class ShellInfoTab(QtGui.QWidget):
         # Add delete button  
         
         t = translate('shell', 'Delete ::: Delete this shell configuration')
-        label = QtGui.QLabel('', self)        
-        instance = QtGui.QPushButton(iep.icons.cancel, t, self)
+        label = QtGui.QLabel('', self._content)        
+        instance = QtGui.QPushButton(iep.icons.cancel, t, self._content)
         instance.setToolTip(t.tt)
         instance.setAutoDefault(False)
         instance.clicked.connect(self.parent().parent().onTabClose)
@@ -350,7 +544,8 @@ class ShellInfoTab(QtGui.QWidget):
         
         # Apply layout
         self._formLayout.setSpacing(15)
-        self.setLayout(self._formLayout)
+        self._content.setLayout(self._formLayout)
+        self.setWidget(self._content)
     
     
     def setTabTitle(self, name):
@@ -391,11 +586,9 @@ class ShellInfoTab(QtGui.QWidget):
         info = self._info
         
         # Set struct values according to widgets
-        try:            
-           for key in info:
-               widget = self._shellInfoWidgets.get(key, None)
-               if widget is not None:
-                   info[key] = widget.getTheText()
+        try:   
+            for key, widget in self._shellInfoWidgets.items():
+                info[key] = widget.getTheText()
         
         except Exception as why:
             print("Error getting info in shell config:", why)
@@ -470,9 +663,8 @@ class ShellInfoDialog(QtGui.QDialog):
         
         # Prevent resizing
         self.show()
-        size = self.size()
-        self.setMinimumSize(size)
-        self.setMaximumHeight(size.height())
+        self.setMinimumSize(500, 400)
+        #self.setMaximumHeight(500)
         
     
     
