@@ -5,24 +5,36 @@
 # The full license can be found in 'license.txt'.
 
 
-""" Module iepRemote2
+""" Module iepkernel.interpreter
 
-Implements the IEP interpreter and the introspection thread.
-Also GUI hijacking is defined here. This code works on all* python versions.
-*: Well, at least from 2.4 and up (including py3k).
+Implements the IEP interpreter.
 
-Note that this module delibirately has a name that is very unlikely to 
-occur in any othe packages to prevent import clashes.
+Notes on IPython
+----------------
+We integrate IPython via the IPython.core.interactiveshell.InteractiveShell.
+  * The namespace is set to __main__
+  * We call its run_cell method to execute code
+  * Debugging/breakpoints are "enabled using the pre_run_code_hook
+  * Debugging occurs in our own debugger
+  * GUI integration is all handled by IEP
+  * We need special prompts for IPython input
+  
+  
 
 """
 
-import os, sys, time
+import os
+import sys
+import time
+import platform
 import struct
+import shlex
 from codeop import CommandCompiler
 import traceback
 import keyword
 import inspect # Must be in this namespace
 import bdb
+from distutils.version import LooseVersion as LV
 
 import yoton
 from iepkernel import guiintegration, printDirect
@@ -34,16 +46,59 @@ sys.last_type = None
 sys.last_value = None
 sys.last_traceback = None
 
-# Set Python version as a float and get some names
-PYTHON_VERSION = sys.version_info[0] + sys.version_info[1]/10.0
+# Set Python version and get some names
+PYTHON_VERSION = sys.version_info[0]
 if PYTHON_VERSION < 3:
     ustr = unicode
     bstr = str
+    input = raw_input
 else:
     ustr = str
     bstr = bytes
 
 
+
+class PS1:
+    """ Dynamic prompt for PS1. Show IPython prompt if available, and
+    show current stack frame when debugging.
+    """
+    def __init__(self, iep):
+        self._iep = iep
+    def __str__(self):
+        if self._iep._dbFrames:
+            # When debugging, show where we are, do not use IPython prompt
+            preamble = '('+self._iep._dbFrameName+')'
+            return '\n\x1b[0;32m%s>>>\x1b[0m ' % preamble
+        elif self._iep._ipython:
+            # IPython prompt
+            return '\n\x1b[0;32mIn [\x1b[1;32m%i\x1b[0;32m]:\x1b[0m ' % (
+                                            self._iep._ipython.execution_count)
+            #return 'In [%i]: ' % (self._ipython.execution_count)
+        else:
+            # Normal Python prompt
+            return '\n\x1b[0;32m>>>\x1b[0m '
+
+
+class PS2:
+    """ Dynamic prompt for PS2.
+    """
+    def __init__(self, iep):
+        self._iep = iep
+    def __str__(self):
+        if self._iep._dbFrames:
+            # When debugging, show where we are, do not use IPython prompt
+            preamble = '('+self._iep._dbFrameName+')'
+            return '\x1b[0;32m%s...\x1b[0m ' % preamble
+        elif self._iep._ipython:
+            # Dots ala IPython
+            nspaces = len(str(self._iep._ipython.execution_count)) + 2
+            return '\x1b[0;32m%s...:\x1b[0m ' % (nspaces*' ')
+        else:
+            # Just dots
+            return '\x1b[0;32m...\x1b[0m '
+
+ 
+
 class IepInterpreter:
     """ IepInterpreter
     
@@ -85,6 +140,10 @@ class IepInterpreter:
         # Store ref of locals that is our main
         self._main_locals = locals
         
+        # Flag to ignore sys exit, to allow running some scripts
+        # interactively, even if they call sys.exit.
+        self.ignore_sys_exit = False
+        
         # Information for debugging. If self._dbFrames, we're in debug mode
         # _dbFrameIndex starts from 1 
         self._dbFrames = []
@@ -108,16 +167,6 @@ class IepInterpreter:
         self.magician = Magician()
         self.debugger = Debugger()
         
-        # Define prompts
-        try:
-            sys.ps1
-        except AttributeError:
-            sys.ps1 = ">>> "
-        try:
-            sys.ps2
-        except AttributeError:
-            sys.ps2 = "... "
-        
         # To keep track of whether to send a new prompt, and whether more
         # code is expected.
         self.more = 0
@@ -197,17 +246,169 @@ class IepInterpreter:
         startup_info['keywords'] = keyword.kwlist
         self.context._stat_startup.send(startup_info)
         
+        # Prepare the Python environment
+        self._prepare_environment(startup_info)
+        
+        # Run startup code (before loading GUI toolkit or IPython
+        self._run_startup_code(startup_info)
+        
         # Write Python banner (to stdout)
         NBITS = 8 * struct.calcsize("P")
-        platform = sys.platform
-        if platform.startswith('win'):
-            platform = 'Windows'
-        platform = '%s (%i bits)' % (platform, NBITS) 
+        plat = sys.platform
+        if plat.startswith('win'):
+            plat = 'Windows'
+        plat = '%s (%i bits)' % (plat, NBITS) 
         printDirect("Python %s on %s.\n" %
-            (sys.version.split('[')[0].rstrip(), platform))
+            (sys.version.split('[')[0].rstrip(), plat))
         
+        # Integrate GUI
+        guiName, guiError = self._integrate_gui(startup_info)
+        
+        # Write IEP part of banner (including what GUI loop is integrated)
+        if True:
+            iepBanner = 'This is the IEP interpreter'
+        if guiError:
+            iepBanner += '. ' + guiError + '\n'
+        elif guiName:
+            iepBanner += ' with integrated event loop for ' 
+            iepBanner += guiName + '.\n'
+        else:
+            iepBanner += '.\n'
+        printDirect(iepBanner)
+        
+        # Try loading IPython
+        if startup_info.get('ipython', '').lower() in ('no', 'false'):
+            self._ipython = None
+        else:
+            try:
+                self._load_ipyhon()
+            except Exception:
+                type, value, tb = sys.exc_info();
+                del tb
+                printDirect('IPython could not be loaded: %s\n' % str(value))
+                self._ipython = None
+        
+        # Set prompts
+        sys.ps1 = PS1(self)
+        sys.ps2 = PS2(self)
+        
+        # Notify about project path
+        projectPath = startup_info['projectPath']
+        if projectPath:
+            printDirect('Prepending the project path %r to sys.path\n' % 
+                projectPath)
+        
+        # Write tips message.
+        if self._ipython:
+            import IPython
+            printDirect("\nUsing IPython %s -- An enhanced Interactive Python.\n"
+                        %  IPython.__version__)
+            printDirect(
+                "?         -> Introduction and overview of IPython's features.\n"
+                "%quickref -> Quick reference.\n"
+                "help      -> Python's own help system.\n"
+                "object?   -> Details about 'object', "
+                "use 'object??' for extra details.\n")
+        else:
+            printDirect("Type 'help' for help, " + 
+                        "type '?' for a list of *magic* commands.\n")
+        
+        # Notify the running of the script
+        if self._scriptToRunOnStartup:
+            printDirect('\x1b[0;33mRunning script: "'+self._scriptToRunOnStartup+'"\x1b[0m\n')
+        
+        # Prevent app nap on OSX 9.2 and up
+        # The _nope module is taken from MINRK's appnope package
+        if sys.platform == "darwin" and LV(platform.mac_ver()[0]) >= LV("10.9"):
+            from iepkernel import _nope
+            _nope.nope()
+    
+    
+    def _prepare_environment(self, startup_info):
+        """ Prepare the Python environment. There are two possibilities:
+        either we run a script or we run interactively.
+        """
+        
+        # Get whether we should (and can) run as script
+        scriptFilename = startup_info['scriptFile']
+        if scriptFilename:
+            if not os.path.isfile(scriptFilename):
+                printDirect('Invalid script file: "'+scriptFilename+'"\n')
+                scriptFilename = None
+        
+        # Get project path
+        projectPath = startup_info['projectPath']
+        
+        # Init script to run on startup
+        self._scriptToRunOnStartup = None
+        
+        if scriptFilename:
+            # RUN AS SCRIPT
+            # Set __file__  (note that __name__ is already '__main__')
+            self.locals['__file__'] = scriptFilename
+            # Set command line arguments
+            sys.argv[:] = []
+            sys.argv.append(scriptFilename)
+            sys.argv.extend(shlex.split(startup_info.get('argv', '')))
+            # Insert script directory to path
+            theDir = os.path.abspath( os.path.dirname(scriptFilename) )
+            if theDir not in sys.path:
+                sys.path.insert(0, theDir)
+            if projectPath is not None:
+                sys.path.insert(0,projectPath)
+            # Go to script dir
+            os.chdir( os.path.dirname(scriptFilename) )
+            # Run script later
+            self._scriptToRunOnStartup = scriptFilename
+        else:
+            # RUN INTERACTIVELY
+            # No __file__ (note that __name__ is already '__main__')
+            self.locals.pop('__file__','')
+            # Remove all command line arguments, set first to empty string
+            sys.argv[:] = []
+            sys.argv.append('')
+            sys.argv.extend(shlex.split(startup_info.get('argv', '')))
+            # Insert current directory to path
+            sys.path.insert(0, '')
+            if projectPath:
+                sys.path.insert(0,projectPath)
+            # Go to start dir
+            startDir = startup_info['startDir']
+            if startDir and os.path.isdir(startDir):
+                os.chdir(startDir)
+            else:
+                os.chdir(os.path.expanduser('~')) # home dir 
+    
+    
+    def _run_startup_code(self, startup_info):
+        """ Execute the startup code or script.
+        """
+        
+        # Run startup script (if set)
+        script = startup_info['startupScript']
+        # Should we use the default startupScript?
+        if script == '$PYTHONSTARTUP':
+            script = os.environ.get('PYTHONSTARTUP','')
+        
+        if '\n' in script:
+            # Run code
+            self.context._stat_interpreter.send('Busy') 
+            msg = {'source': script, 'fname': '<startup>', 'lineno': 0}
+            self.runlargecode(msg, True)
+        elif script and os.path.isfile(script):
+            # Run script
+            self.context._stat_interpreter.send('Busy') 
+            self.runfile(script)
+        else:
+            # Nothing to run
+            pass
+    
+    
+    def _integrate_gui(self, startup_info):
+        """ Integrate event loop of GUI toolkit (or use pure Python
+        event loop).
+        """
         
-        # Integrate event loop of GUI toolkit
         self.guiApp = guiintegration.App_base()
         self.guiName = guiName = startup_info['gui'].upper()
         guiError = ''
@@ -236,93 +437,60 @@ class IepInterpreter:
             guiError = 'Failed to integrate event loop for %s: %s' % (
                 guiName, str(value))
         
-        # Write IEP part of banner (including what GUI loop is integrated)
-        if True:
-            iepBanner = 'This is the IEP interpreter'
-        if guiError:
-            iepBanner += '. ' + guiError + '\n'
-        elif guiName:
-            iepBanner += ' with integrated event loop for ' 
-            iepBanner += guiName + '.\n'
+        return guiName, guiError
+    
+    
+    def _load_ipyhon(self):
+        """ Try loading IPython shell. The result is set in self._ipython
+        (can be None if IPython not available).
+        """
+        
+        # Init
+        self._ipython = None
+        import __main__
+        
+        # Try importing IPython
+        try:
+            import IPython
+        except ImportError:
+            return
+        
+        # Version ok?
+        if IPython.version_info < (1,):
+            return
+        
+        # Create an IPython shell
+        from IPython.core.interactiveshell import InteractiveShell 
+        self._ipython = InteractiveShell(user_module=__main__)
+        
+        # Set some hooks / event callbacks
+        # Run hook (pre_run_code_hook is depreacted in 2.0)
+        pre_run_cell_hook  = self.ipython_pre_run_cell_hook
+        if IPython.version_info < (2,):
+            self._ipython.set_hook('pre_run_code_hook', pre_run_cell_hook)
         else:
-            iepBanner += '.\n'
-        printDirect(iepBanner)
+            self._ipython.events.register('pre_run_cell', pre_run_cell_hook)
+        # Other hooks
+        self._ipython.set_hook('editor', self.ipython_editor_hook)
+        self._ipython.set_custom_exc((bdb.BdbQuit,), self.dbstop_handler)
         
+        # Some patching
+        self._ipython.ask_exit = self.ipython_ask_exit
         
-        # Append project path if given
-        projectPath = startup_info['projectPath']
-        if projectPath:
-            printDirect('Prepending the project path %r to sys.path\n' % 
-                projectPath)
-            #Actual prepending is done below, to put it before the script path
-        
-        # Write tips message
-        printDirect('Type "help" for help, ' + 
-                            'type "?" for a list of *magic* commands.\n')
-        
-        
-        # Get whether we should (and can) run as script
-        scriptFilename = startup_info['scriptFile']
-        if scriptFilename:
-            if not os.path.isfile(scriptFilename):
-                printDirect('Invalid script file: "'+scriptFilename+'"\n')
-                scriptFilename = None
-        
-        # Init script to run on startup
-        self._scriptToRunOnStartup = None
-        
-        if scriptFilename:
-            # RUN AS SCRIPT
+        # Make output be shown on Windows
+        if sys.platform.startswith('win'):
+            # IPython wraps std streams just like we do below, but
+            # pyreadline adds *another* wrapper, which is where it
+            # goes wrong. Here we set it back to bypass pyreadline.
+            from IPython.utils import io
+            io.stdin = io.IOStream(sys.stdin)
+            io.stdout = io.IOStream(sys.stdout)
+            io.stderr = io.IOStream(sys.stderr)
             
-            # Set __file__  (note that __name__ is already '__main__')
-            self.locals['__file__'] = scriptFilename
-            # Set command line arguments
-            sys.argv[:] = []
-            sys.argv.append(scriptFilename)
-            # Insert script directory to path
-            theDir = os.path.abspath( os.path.dirname(scriptFilename) )
-            if theDir not in sys.path:
-                sys.path.insert(0, theDir)
-            if projectPath is not None:
-                sys.path.insert(0,projectPath)
-            
-            # Go to script dir
-            os.chdir( os.path.dirname(scriptFilename) )
-            
-            # Notify the running of the script
-            printDirect('[Running script: "'+scriptFilename+'"]\n')
-            
-            # Run script
-            self._scriptToRunOnStartup = scriptFilename
-        
-        else:
-            # RUN INTERACTIVELY
-            
-            # No __file__ (note that __name__ is already '__main__')
-            self.locals.pop('__file__','')
-            # Remove all command line arguments, set first to empty string
-            sys.argv[:] = []
-            sys.argv.append('')
-            # Insert current directory to path
-            sys.path.insert(0, '')
-            if projectPath:
-                sys.path.insert(0,projectPath)
-                
-            # Go to start dir
-            startDir = startup_info['startDir']
-            if startDir and os.path.isdir(startDir):
-                os.chdir(startDir)
-            else:
-                os.chdir(os.path.expanduser('~')) # home dir 
-            
-            # Run startup script (if set)
-            filename = startup_info['startupScript']
-            # Should we use the default startupScript?
-            if filename == '$PYTHONSTARTUP':
-                filename = os.environ.get('PYTHONSTARTUP','')
-            # Check if it exists
-            if filename and os.path.isfile(filename):
-                self._scriptToRunOnStartup = filename
+            # Ipython uses msvcrt e.g. for pausing between pages
+            # but this does not work in IEP
+            import msvcrt
+            msvcrt.getwch = msvcrt.getch = input  # input is deffed above
     
     
     def process_commands(self):
@@ -331,21 +499,12 @@ class IepInterpreter:
         try:
             
             self._process_commands()
-        
-        except KeyboardInterrupt:
-            self.write("\nKeyboardInterrupt\n")
-            self._resetbuffer()
-            self.more = 0
-        except TypeError:
-            # For some reason, when wx is integrated, keyboard interrupts
-            # result in a TypeError.
-            # I tried to find the source, but did not find it. If anyone
-            # has an idea, please e-mail me!
-            if self.guiName == 'WX':
-                self.write("\nKeyboard Interrupt\n") # space to see difference
-                self._resetbuffer()
-                self.more = 0
+            
         except SystemExit:
+            # It may be that we should ignore sys exit now...
+            if self.ignore_sys_exit:
+                self.ignore_sys_exit = False  # Never allow more than once
+                return
             # Get and store the exception so we can raise it later
             type, value, tb = sys.exc_info();  del tb
             self._exitException = value
@@ -368,14 +527,8 @@ class IepInterpreter:
         # Prompt is allowed to be an object with __str__ method
         if self.newPrompt:
             self.newPrompt = False
-            # Write prompt (note that the second "if" is not an "elif"!
-            preamble = ''
-            if self._dbFrames:
-                preamble = '('+self._dbFrameName+')'
-            if self.more:
-                self.context._strm_prompt.send(preamble+str(sys.ps2))
-            else:
-                self.context._strm_prompt.send(preamble+str(sys.ps1))
+            ps = sys.ps2 if self.more else sys.ps1
+            self.context._strm_prompt.send(str(ps))
         
         if True:
             # Determine state. The message is really only send
@@ -410,11 +563,12 @@ class IepInterpreter:
                 self.context._strm_echo.send(line1)
                 self.context._stat_interpreter.send('Busy')
                 self.newPrompt = True
-                # Convert command
+                # Convert command 
+                # (only a few magics are supported if IPython is active)
                 line2 = self.magician.convert_command(line1.rstrip('\n'))
                 # Execute actual code
                 if line2 is not None:
-                    for line3 in line2.split('\n'): # not splitlines!
+                    for line3 in line2.split('\n'):  # not splitlines!
                         self.more = self.pushline(line3)
                 else:
                     self.more = False
@@ -462,14 +616,19 @@ class IepInterpreter:
         with in some way (this is the same as _runlines()).
         
         """
-        self._buffer.append(line)
-        source = "\n".join(self._buffer)
+        # Get buffer, join to get source
+        buffer = self._buffer
+        buffer.append(line)
+        source = "\n".join(buffer)
+        # Clear buffer and run source
+        self._resetbuffer()
         more = self._runlines(source, self._filename)
-        if not more:
-            self._resetbuffer()
+        # Create buffer if needed
+        if more:
+            self._buffer = buffer 
         return more
     
-    
+
     def _runlines(self, source, filename="<input>", symbol="single"):
         """Compile and run some source in the interpreter.
         
@@ -494,23 +653,42 @@ class IepInterpreter:
         line.
         
         """
+        
+        use_ipython = self._ipython and not self._dbFrames
+        
+        # Try compiling.
+        # The IPython kernel does not handle incomple lines, so we check
+        # that ourselves ...
         try:
             code = self.compilecode(source, filename, symbol)
         except (OverflowError, SyntaxError, ValueError):
-            # Case 1
-            self.showsyntaxerror(filename)
-            return False
+            code = False
         
-        if code is None:
-            # Case 2
-            return True
-        
-        # Case 3
-        self.execcode(code)
-        return False
+        if use_ipython:
+            if code is None:
+                # Case 2
+                #self._ipython.run_cell('', True)
+                return True
+            else:
+                # Case 1 and 3 handled by IPython
+                self._ipython.run_cell(source, True, False)
+                return False
+                
+        else:
+            if code is None:
+                # Case 2
+                return True
+            elif not code:
+                # Case 1
+                self.showsyntaxerror(filename)
+                return False
+            else:
+                # Case 3
+                self.execcode(code)
+                return False
     
     
-    def runlargecode(self, msg):
+    def runlargecode(self, msg, silent=False):
         """ To execute larger pieces of code. """
         
         # Get information
@@ -532,7 +710,11 @@ class IepInterpreter:
             runtext = '(executing lines %i to %i of "%s")\n' % (
                                                 lineno1, lineno2, fname_show)
         # Notify IDE
-        self.context._strm_echo.send(runtext)
+        if not silent:
+            self.context._strm_echo.send('\x1b[0;33m%s\x1b[0m' % runtext)
+            # Increase counter
+            if self._ipython:
+                self._ipython.execution_count += 1
         
         # Put the line number in the filename (if necessary)
         # Note that we could store the line offset in the _codeCollection,
@@ -552,7 +734,7 @@ class IepInterpreter:
         
         if code:
             # Store the source using the (id of the) code object as a key
-            self._codeCollection.storeSource(code, source)
+            self._codeCollection.store_source(code, source)
             # Execute the code
             self.execcode(code)
         else:
@@ -589,7 +771,7 @@ class IepInterpreter:
         
         if code:
             # Store the source using the (id of the) code object as a key
-            self._codeCollection.storeSource(code, source)
+            self._codeCollection.store_source(code, source)
             # Execute the code
             self.execcode(code)
         else:
@@ -645,9 +827,30 @@ class IepInterpreter:
         The globals variable is used when in debug mode.
         """
         
-        # Apply breakpoints
-        # So breakpoints are updated at each time a command is given,
-        # including commands like "db continue".
+        try:
+            if self._dbFrames:
+                self.apply_breakpoints()
+                exec(code, self.globals, self.locals)
+            else:
+                # Turn debugger on at this point. If there are no breakpoints,
+                # the tracing is disabled for better performance.
+                self.apply_breakpoints()
+                self.debugger.set_on() 
+                exec(code, self.locals)
+        except bdb.BdbQuit:
+            self.dbstop_handler()
+        except Exception:
+            time.sleep(0.2) # Give stdout some time to send data
+            self.showtraceback()
+        except KeyboardInterrupt: # is a BaseException, not an Exception
+            time.sleep(0.2)
+            self.showtraceback()
+    
+    
+    def apply_breakpoints(self):
+        """ Breakpoints are updated at each time a command is given,
+        including commands like "db continue".
+        """
         try:
             breaks = self.context._stat_breakpoints.recv()
             if self.debugger.breaks:
@@ -659,24 +862,44 @@ class IepInterpreter:
         except Exception:
             type, value, tb = sys.exc_info(); del tb
             print('Error while setting breakpoints: %s' % str(value))
-        
-        
-        try:
-            if self._dbFrames:
-                exec(code, self.globals, self.locals)
-            else:
-                # Turn debugger on at this point. If there are no breakpoints,
-                # the tracing is disabled for better performance.
-                self.debugger.set_on() 
-                exec(code, self.locals)
-        except bdb.BdbQuit:
-            print("Program execution stopped from debugger.")
-        except Exception:
-            time.sleep(0.2) # Give stdout some time to send data
-            self.showtraceback()
-        except KeyboardInterrupt: # is a BaseException, not an Exception
-            time.sleep(0.2)
-            self.showtraceback()
+    
+    
+    ## Handlers and hooks
+    
+    def ipython_pre_run_cell_hook(self, ipython=None):
+        """ Hook that IPython calls right before executing code.
+        """
+        self.apply_breakpoints()
+        self.debugger.set_on() 
+    
+    
+    def ipython_editor_hook(self, ipython, filename, linenum=None, wait=True):
+        # Correct line number for cell offset
+        filename, linenum = self.correctfilenameandlineno(filename, linenum or 0)
+        # Get action string
+        if linenum:
+            action = 'open %i %s' % (linenum, os.path.abspath(filename))
+        else:
+            action = 'open %s' % os.path.abspath(filename)
+        # Send
+        self.context._strm_action.send(action)
+    
+    
+    def ipython_ask_exit(self):
+        # Ask the user
+        a = input("Do you really want to exit ([y]/n)? ")
+        a = a or 'y'
+        # Close stdin if necessary
+        if a.lower() == 'y':
+            sys.stdin._channel.close()
+    
+    
+    def dbstop_handler(self, *args, **kwargs):
+        print("Program execution stopped from debugger.")
+    
+    
+    
+    
     
     
     ## Writing and error handling
@@ -772,7 +995,7 @@ class IepInterpreter:
                     sys.last_value = value
                     sys.last_traceback = tb
             
-            # Get tpraceback to correct all the line numbers
+            # Get traceback to correct all the line numbers
             # tblist = list  of (filename, line-number, function-name, text)
             tblist = traceback.extract_tb(tb)
             
@@ -781,7 +1004,6 @@ class IepInterpreter:
             while tb:
                 frames.append(tb.tb_frame)
                 tb = tb.tb_next
-            frames.pop(0)
             
             # Walk through the list
             for i in range(len(tblist)):
@@ -791,17 +1013,6 @@ class IepInterpreter:
                 if not isinstance(fname, ustr):
                     fname = fname.decode('utf-8')
                 example = tbInfo[3]
-                # Get source (if available) and split lines
-                source = None
-                if i < len(frames):
-                    source = self._codeCollection.getSource(frames[i].f_code)
-                if source:
-                    source = source.splitlines()                
-                    # Obtain source from example and select line                    
-                    try:
-                        example = source[ tbInfo[1]-1 ]
-                    except IndexError:
-                        pass
                 # Reset info
                 tblist[i] = (fname, lineno, tbInfo[2], example)
             
@@ -831,7 +1042,7 @@ class IepInterpreter:
         As example:
         "foo.py+7", 22  -> "foo.py", 29
         """
-        j = fname.find('+')
+        j = fname.rfind('+')
         if j>0:
             try:
                 lineno += int(fname[j+1:])
@@ -841,15 +1052,50 @@ class IepInterpreter:
         return fname, lineno
 
 
-class ExecutedSourceCollection(dict):
-    """ Stores the source of executed pieces of code, so that the right 
-    traceback can be reproduced when an error occurs.
-    The codeObject produced by compiling the source is used as a 
-    reference.
+class ExecutedSourceCollection:
+    """ Stores the source of executed pieces of code, so that the right
+    traceback can be reproduced when an error occurs. The filename
+    (including the +lineno suffix) is used as a key. We monkey-patch
+    the linecache module so that we first try our cache to look up the
+    lines. In that way we also allow third party modules (e.g. IPython)
+    to get the lines for executed cells.
     """
-    def _getId(self, codeObject):
-        id_ = str(id(codeObject)) + '_' + codeObject.co_filename
-    def storeSource(self, codeObject, source):
-        self[self._getId(codeObject)] = source
-    def getSource(self, codeObject):
-        return self.get(self._getId(codeObject), '')
+    
+    def __init__(self):
+        self._cache = {}
+        self._patch()
+    
+    def store_source(self, codeObject, source):
+        self._cache[codeObject.co_filename] = source
+    
+    def _patch(self):
+        def getlines(filename, module_globals=None):
+            # Try getting the source from our own cache,
+            # otherwise fallback to linecache's own cache
+            src = self._cache.get(filename, '')
+            if src:
+                return [line+'\n' for line in src.splitlines()]
+            else:
+                import linecache
+                return linecache._getlines(filename, module_globals)
+        
+        # Monkey patch
+        import linecache
+        linecache._getlines = linecache.getlines
+        linecache.getlines =getlines
+        
+        # I hoped this would remove the +lineno for IPython tracebacks, 
+        # but it doesn't
+#         def extract_tb(tb, limit=None):
+#             print('aasdasd')
+#             import traceback
+#             list1 = traceback._extract_tb(tb, limit)
+#             list2 = []
+#             for (filename, lineno, name, line) in list1:
+#                 filename, lineno = sys._iepInterpreter.correctfilenameandlineno(filename, lineno)
+#                 list2.append((filename, lineno, name, line))
+#             return list2
+#         
+#         import traceback
+#         traceback._extract_tb = traceback.extract_tb
+#         traceback.extract_tb = extract_tb
