@@ -809,9 +809,18 @@ class PopupMenu(iep.iepcore.menu.Menu):
             if True:
                 self.addItem(translate("filebrowser", "Copy path"), 
                     None, self._copyPath)
+                    
         
             self.addSeparator()
+            
+        # Import data wizard
+        if isinstance(self._item, FileItem):
+            self.addItem(translate("filebrowser", "Import data..."),
+                    None, self._importData)
         
+            self.addSeparator()     
+            
+             
         # Create items for file management
         if isinstance(self._item, FileItem):
             self.addItem(translate("filebrowser", "Rename"), None, self.onRename)
@@ -822,6 +831,7 @@ class PopupMenu(iep.iepcore.menu.Menu):
             self.addItem(translate("filebrowser", "Create new directory"), None, self.onCreateDir)
         if isinstance(self._item, DirItem):
             self.addSeparator()
+            self.addItem(translate("filebrowser", "Rename"), None, self.onRename)
             self.addItem(translate("filebrowser", "Delete"), None, self.onDelete)
     
     
@@ -852,6 +862,10 @@ class PopupMenu(iep.iepcore.menu.Menu):
     def _copyPath(self):
         QtGui.qApp.clipboard().setText(self._item.path())
     
+    def _importData(self):
+        browser = self.parent().parent()
+        wizard = browser.getImportWizard()
+        wizard.open(self._item.path())
     
     def onDuplicate(self):
         return self._duplicateOrRename(False)
